@@ -28,15 +28,9 @@
 #include "handlers/gda-handler-time.h"
 #include "handlers/gda-handler-type.h"
 
-#if GLIB_CHECK_VERSION(2,31,7)
 static GRecMutex init_rmutex;
 #define MUTEX_LOCK() g_rec_mutex_lock(&init_rmutex)
 #define MUTEX_UNLOCK() g_rec_mutex_unlock(&init_rmutex)
-#else
-static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
-#define MUTEX_LOCK() g_static_rec_mutex_lock(&init_mutex)
-#define MUTEX_UNLOCK() g_static_rec_mutex_unlock(&init_mutex)
-#endif
 static void gda_data_handler_iface_init (gpointer g_class);
 
 GType
@@ -307,11 +301,11 @@ gtype_equal (gconstpointer a, gconstpointer b)
 GdaDataHandler *
 gda_data_handler_get_default (GType for_type)
 {
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+	static GMutex mutex;
 	static GHashTable *hash = NULL;
 	GdaDataHandler *dh;
 
-	g_static_mutex_lock (&mutex);
+	g_mutex_lock (&mutex);
 	if (!hash) {
 		hash = g_hash_table_new_full (gtype_hash, gtype_equal,
 					      NULL, (GDestroyNotify) g_object_unref);
@@ -339,7 +333,7 @@ gda_data_handler_get_default (GType for_type)
                 g_hash_table_insert (hash, (gpointer) G_TYPE_GTYPE, gda_handler_type_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_UINT, gda_handler_numerical_new ());
 	}
-	g_static_mutex_unlock (&mutex);
+	g_mutex_unlock (&mutex);
 
 	dh = g_hash_table_lookup (hash, (gpointer) for_type);
 	return dh;
