@@ -2,7 +2,7 @@
  * Copyright (C) 2001 - 2003 Rodrigo Moya <rodrigo@gnome-db.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
  * Copyright (C) 2002 Holger Thon <holger.thon@gnome-db.org>
- * Copyright (C) 2002 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2002 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
  * Copyright (C) 2003 Akira TAGOH <tagoh@gnome-db.org>
  * Copyright (C) 2003 Danilo Schoeneberg <dschoene@src.gnome.org>
@@ -172,11 +172,7 @@ set_from_string (GValue *value, const gchar *as_string)
 	else if (type == G_TYPE_CHAR) {
 		lvalue = strtol (as_string, endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
-#if GLIB_CHECK_VERSION(2,31,7)
 			g_value_set_schar(value,(gint)lvalue);
-#else
-			g_value_set_char(value,(gchar)lvalue);
-#endif
 			retval = TRUE;
 		}
 	}
@@ -281,9 +277,7 @@ set_from_string (GValue *value, const gchar *as_string)
 			retval = TRUE;
 		}
 	}
-
-
-	if (!retval && g_value_type_transformable (G_TYPE_STRING, type)) {
+	else if (g_value_type_transformable (G_TYPE_STRING, type)) {
 		/* use the GLib type transformation function */
 		GValue *string;
 
@@ -415,7 +409,6 @@ gda_default_get_type (void)
 static void
 string_to_binary (const GValue *src, GValue *dest)
 {
-	/* FIXME: add more checks*/
 	GdaBinary *bin;
 	const gchar *as_string;
 
@@ -513,7 +506,6 @@ gda_binary_free (gpointer boxed)
 static void
 string_to_blob (const GValue *src, GValue *dest)
 {
-	/* FIXME: add more checks*/
 	GdaBlob *blob;
 	const gchar *as_string;
 
@@ -656,7 +648,6 @@ geometric_point_to_string (const GValue *src, GValue *dest)
 static void
 string_to_geometricpoint (const GValue *src, GValue *dest)
 {
-	/* FIXME: add more checks*/
 	GdaGeometricPoint *point;
 	const gchar *as_string;
 
@@ -867,13 +858,19 @@ GdaNumeric*
 gda_numeric_copy (GdaNumeric *src)
 {
 	GdaNumeric *copy;
+	gchar *str;
 
 	g_return_val_if_fail (src, NULL);
 
 	copy = gda_numeric_new ();
-	gda_numeric_set_from_string (copy, gda_numeric_get_string (src));
+
+	str = gda_numeric_get_string (src);
+	gda_numeric_set_from_string (copy, str);
+	g_free (str);
+
 	gda_numeric_set_width (copy, gda_numeric_get_width (src));
 	gda_numeric_set_precision (copy, gda_numeric_get_precision (src));
+
 	return copy;
 }
 
@@ -1238,14 +1235,65 @@ gda_time_valid (const GdaTime *time)
 {
 	g_return_val_if_fail (time, FALSE);
 
-	if (time->hour > 24 ||
-	    time->minute > 59 ||
-	    time->second > 59)
+	if ((time->hour > 23) ||
+	    (time->minute > 59) ||
+	    (time->second > 59))
 		return FALSE;
-	else
-		return TRUE;
+	if ((time->fraction >= 1000000) ||
+	    (time->timezone <= -12 * 3600) ||
+	    (time->timezone >= 12 * 3600))
+		return FALSE;
+	return TRUE;
 }
 
+/**
+ * gda_time_change_timezone:
+ * @time: a valid #GdaTime
+ * @ntz: a new timezone to use, in seconds added to GMT
+ *
+ * Changes @time's timezone (for example to convert from GMT to another time zone).
+ * If @time's current timezone is unset (i.e. equal to %GDA_TIMEZONE_INVALID), then this function simply sets
+ * @time's timezone attribute; Otherwise, it adds or removes hours, minutes or seconds to reflect the time in the new timezone.
+ *
+ * Note: the resulting will always be a valid time.
+ *
+ * Since: 5.2
+ */
+void
+gda_time_change_timezone (GdaTime *time, glong ntz)
+{
+	g_return_if_fail (time);
+	g_return_if_fail (gda_time_valid (time));
+	g_return_if_fail ((ntz > - 12 * 3600) && (ntz < 12 * 3600));
+
+	if (time->timezone == ntz)
+		return;
+
+	if (time->timezone != GDA_TIMEZONE_INVALID) {
+		glong nsec;
+		nsec = time->hour * 3600 + time->minute * 60 + time->second - time->timezone + ntz;
+		if (nsec < 0)
+			nsec += 86400;
+		else if (nsec >= 86400)
+			nsec -= 86400;
+
+		/* hours */
+		gint n;
+		n = nsec / 3600;
+		time->hour = (gushort) n;
+
+		/* minutes */
+		nsec -= n * 3600;
+		n = nsec / 60;
+		time->minute = (gushort) n;
+
+		/* seconds */
+		nsec -= n * 60;
+		time->second = (gushort) nsec;
+	}
+
+	time->timezone = ntz;
+}
 
 /*
  * Register the GdaTimestamp type in the GType system
@@ -1254,14 +1302,12 @@ gda_time_valid (const GdaTime *time)
 static void
 string_to_timestamp (const GValue *src, GValue *dest)
 {
-	/* FIXME: add more checks*/
 	GdaTimestamp timestamp;
 
 	g_return_if_fail (G_VALUE_HOLDS_STRING (src) &&
 			  GDA_VALUE_HOLDS_TIMESTAMP (dest));
 
-	if (! gda_parse_iso8601_timestamp (&timestamp, g_value_get_string (src)))
-		g_warning ("Can't convert '%s' to a timestamp", g_value_get_string (src));
+	gda_parse_iso8601_timestamp (&timestamp, g_value_get_string (src));
 	gda_value_set_timestamp (dest, &timestamp);
 }
 
@@ -1358,26 +1404,85 @@ gda_timestamp_valid (const GdaTimestamp *timestamp)
 {
 	g_return_val_if_fail (timestamp, FALSE);
 
-	GDate *gdate;
-
 	/* check the date part */
-	gdate = g_date_new_dmy ((GDateDay) timestamp->day, (GDateMonth) timestamp->month,
-				(GDateYear) timestamp->year);
-	if (!gdate)
+	if (! g_date_valid_dmy ((GDateDay) timestamp->day, (GDateMonth) timestamp->month,
+				(GDateYear) timestamp->year))
 		return FALSE;
-	if (! g_date_valid (gdate)) {
-		g_date_free (gdate);
-		return FALSE;
-	}
-	g_date_free (gdate);
 
 	/* check the time part */
-	if (timestamp->hour > 24 ||
-	    timestamp->minute > 59 ||
-	    timestamp->second > 59)
+	if ((timestamp->hour > 23) ||
+	    (timestamp->minute > 59) ||
+	    (timestamp->second > 59))
 		return FALSE;
-	else
-		return TRUE;
+	if ((timestamp->fraction >= 1000000) ||
+	    (timestamp->timezone <= -12 * 3600) ||
+	    (timestamp->timezone >= 12 * 3600))
+		return FALSE;
+
+	return TRUE;
+}
+
+/**
+ * gda_timestamp_change_timezone:
+ * @ts: a valid #GdaTimestamp
+ * @ntz: a new timezone to use, in seconds added to GMT
+ *
+ * This function is similar to gda_time_change_timezone() but operates on time stamps.
+ *
+ * Note: the resulting will always be a valid time.
+ *
+ * Since: 5.2
+ */
+void
+gda_timestamp_change_timezone (GdaTimestamp *ts, glong ntz)
+{
+	g_return_if_fail (ts);
+	g_return_if_fail (gda_timestamp_valid (ts));
+	g_return_if_fail ((ntz > - 12 * 3600) && (ntz < 12 * 3600));
+
+	if (ts->timezone == ntz)
+		return;
+
+	if (ts->timezone != GDA_TIMEZONE_INVALID) {
+		glong nsec;
+		nsec = ts->hour * 3600 + ts->minute * 60 + ts->second - ts->timezone + ntz;
+		if (nsec < 0) {
+			GDate *date;
+			date = g_date_new_dmy ((GDateDay) ts->day, (GDateMonth) ts->month, (GDateYear) ts->year);
+			g_date_subtract_days (date, 1);
+			ts->year = g_date_get_year (date);
+			ts->month = g_date_get_month (date);
+			ts->day = g_date_get_day (date);
+			g_date_free (date);
+			nsec += 86400;
+		}
+		else if (nsec >= 86400) {
+			GDate *date;
+			date = g_date_new_dmy ((GDateDay) ts->day, (GDateMonth) ts->month, (GDateYear) ts->year);
+			g_date_add_days (date, 1);
+			ts->year = g_date_get_year (date);
+			ts->month = g_date_get_month (date);
+			ts->day = g_date_get_day (date);
+			g_date_free (date);
+			nsec -= 86400;
+		}
+
+		/* hours */
+		gint n;
+		n = nsec / 3600;
+		ts->hour = (gushort) n;
+
+		/* minutes */
+		nsec -= n * 3600;
+		n = nsec / 60;
+		ts->minute = (gushort) n;
+
+		/* seconds */
+		nsec -= n * 60;
+		ts->second = (gushort) nsec;
+	}
+
+	ts->timezone = ntz;
 }
 
 /**
@@ -1527,7 +1632,14 @@ gda_value_new_blob_from_file (const gchar *filename)
  * @val: value to set for the new #GValue.
  *
  * Makes a new #GValue of type #GDA_TYPE_TIMESTAMP with value @val
- * (of type time_t).
+ * (of type time_t). The returned timestamp has a timezone initialized with the
+ * current timezone, taking into account the daylight savings.
+ *
+ * For example, to get a time stamp representing the current date and time, use:
+ *
+ * <code>
+ * ts = gda_value_new_timestamp_from_timet (time (NULL));
+ * </code>
  *
  * Returns: (transfer full): the newly created #GValue.
  *
@@ -1543,6 +1655,7 @@ gda_value_new_timestamp_from_timet (time_t val)
         value = g_new0 (GValue, 1);
 #ifdef HAVE_LOCALTIME_R
 	struct tm tmpstm;
+	tzset ();
 	ltm = localtime_r ((const time_t *) &val, &tmpstm);
 #elif HAVE_LOCALTIME_S
 	struct tm tmpstm;
@@ -1563,7 +1676,7 @@ gda_value_new_timestamp_from_timet (time_t val)
                 tstamp.minute = ltm->tm_min;
                 tstamp.second = ltm->tm_sec;
                 tstamp.fraction = 0;
-                tstamp.timezone = GDA_TIMEZONE_INVALID;
+                tstamp.timezone = - timezone + daylight * 3600;
                 gda_value_set_timestamp (value, (const GdaTimestamp *) &tstamp);
         }
 
@@ -2169,8 +2282,10 @@ gda_value_stringify (const GValue *value)
 		GValue *string;
 		gchar *str;
 
+		setlocale (LC_NUMERIC, "C");
 		string = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
 		g_value_transform (value, string);
+		setlocale (LC_NUMERIC, gda_numeric_locale);
 		str = g_value_dup_string (string);
 		gda_value_free (string);
 		return str;
@@ -2595,13 +2710,8 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	}
 
 	else if (type == G_TYPE_CHAR) {
-#if GLIB_CHECK_VERSION(2,31,7)
 		gint8 c1 = g_value_get_schar (value1);
 		gint8 c2 = g_value_get_schar (value2);
-#else
-		gchar c1 = g_value_get_char (value1);
-		gchar c2 = g_value_get_char (value2);
-#endif
 		return (c1 > c2) ? 1 : ((c1 == c2) ? 0 : -1);
 	}
 
