@@ -23,9 +23,11 @@
  */
 
 #include "gda-handler-time.h"
+#include <gda-util.h>
 #include <string.h>
 #include <ctype.h>
 #include <glib/gi18n-lib.h>
+#include <libgda/gda-debug-macros.h>
 
 static void gda_handler_time_class_init (GdaHandlerTimeClass *class);
 static void gda_handler_time_init (GdaHandlerTime *hdl);
@@ -87,7 +89,7 @@ gda_handler_time_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerTimeClass),
 			(GBaseInitFunc) NULL,
@@ -107,12 +109,12 @@ gda_handler_time_get_type (void)
 			NULL
 		};
 
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0) {
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerTime", &info, 0);
 			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
 		}
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -452,12 +454,19 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *dh, const GValue
 	}
 	else if (type == GDA_TYPE_TIME) {
 		const GdaTime *tim;
-		
+		GString *string;
+		string = g_string_new ("");
+		g_string_append_c (string, '\'');
 		tim = gda_value_get_time ((GValue *) value);
-		retval = g_strdup_printf ("'%02d:%02d:%02d'",
-					  tim->hour,
-					  tim->minute,
-					  tim->second);
+		g_string_append_printf (string, "%02d:%02d:%02d",
+					tim->hour,
+					tim->minute,
+					tim->second);
+		if (tim->timezone != GDA_TIMEZONE_INVALID)
+			g_string_append_printf (string, "%+02d",
+						(int) tim->timezone / 3600);
+		g_string_append_c (string, '\'');
+		retval = g_string_free (string, FALSE);
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		const GdaTimestamp *gdats;
@@ -479,7 +488,7 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *dh, const GValue
 				g_string_append_printf (string, ".%lu", gdats->fraction);
 			
 			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d", 
+				g_string_append_printf (string, "%+02d",
 							(int) gdats->timezone / 3600);
 			
 			retval = g_strdup_printf ("%s %s", str, string->str);
@@ -620,12 +629,19 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 	}
 	else if (type == GDA_TYPE_TIME) {
 		const GdaTime *tim;
-
+		GString *string;
+		string = g_string_new ("");
 		tim = gda_value_get_time ((GValue *) value);
-		retval = g_strdup_printf ("'%02d:%02d:%02d'",
-					  tim->hour,
-					  tim->minute,
-					  tim->second);
+		g_string_append_c (string, '\'');
+		g_string_append_printf (string, "%02d:%02d:%02d",
+					tim->hour,
+					tim->minute,
+					tim->second);
+		if (tim->timezone != GDA_TIMEZONE_INVALID)
+			g_string_append_printf (string, "%+02d",
+						(int) tim->timezone / 3600);
+		g_string_append_c (string, '\'');
+		retval = g_string_free (string, FALSE);
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		const GdaTimestamp *gdats;
@@ -647,7 +663,7 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 				g_string_append_printf (string, ".%lu", gdats->fraction);
 			
 			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d", 
+				g_string_append_printf (string, "%+02d",
 							(int) gdats->timezone / 3600);
 			
 			retval = g_strdup_printf ("'%s %s'", str, string->str);
@@ -751,7 +767,7 @@ gda_handler_time_get_str_from_value (GdaDataHandler *iface, const GValue *value)
 				g_string_append_printf (string, ".%lu", gdats->fraction);
 			
 			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d", 
+				g_string_append_printf (string, "%+02d",
 							(int) gdats->timezone / 3600);
 			
 			retval = g_strdup_printf ("%s %s", str, string->str);
@@ -1193,111 +1209,22 @@ make_date (G_GNUC_UNUSED GdaHandlerTime *hdl, GDate *date, const gchar *value,
 static gboolean
 make_time (G_GNUC_UNUSED GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value)
 {
-	const gchar *ptr;
-
-	memset (timegda, 0, sizeof (GdaTime));
-	timegda->timezone = GDA_TIMEZONE_INVALID;
-
-	if (!value)
-		return FALSE;
-
-	/* hour */
-	ptr = value;
-	if (!isdigit (*ptr))
-		return FALSE;
-
-	timegda->hour = *ptr - '0';
-	ptr++;
-	if (isdigit (*ptr)) {
-		timegda->hour = timegda->hour * 10 + (*ptr - '0');
-		ptr++;
-	}
-	if (timegda->hour >= 24)
-		return FALSE;
-
-	if (*ptr == ':')
-		ptr++;
-	else if (!isdigit (*ptr))
-		return FALSE;
-
-	/* minute */
-	timegda->minute = *ptr - '0';
-	ptr++;
-	if (isdigit (*ptr)) {
-		timegda->minute = timegda->minute * 10 + (*ptr - '0');
-		ptr++;
-	}
-	if (timegda->minute >= 60)
-		return FALSE;
-	if (*ptr == ':')
-		ptr++;
-	else if (!isdigit (*ptr))
-		return FALSE;
-
-	/* second */
-	timegda->second = *ptr - '0';
-	ptr++;
-	if (isdigit (*ptr)) {
-		timegda->second = timegda->second * 10 + (*ptr - '0');
-		ptr++;
-	}
-	if (timegda->second >= 60)
-		return FALSE;
-
-	/* fraction */
-	if (*ptr && (*ptr != '.') && (*ptr != '+') && (*ptr != '-'))
-		return FALSE;
-	if (*ptr == '.') {
-		ptr++;
-		if (!isdigit (*ptr))
-			return FALSE;
-		while (isdigit (*ptr)) {
-			unsigned long long lu;
-			lu = timegda->fraction * 10 + (*ptr - '0');
-			if (lu < G_MAXULONG)
-				timegda->fraction = lu;
-			else
-				return FALSE;
-			ptr++;
-		}
-	}
-	
-	/* timezone */
-	if ((*ptr == '-') || (*ptr == '+')) {
-		gint sign = (*ptr == '+') ? 1 : -1;
-		ptr++;
-		if (!isdigit (*ptr))
-			return FALSE;
-		timegda->timezone = sign * (*ptr - '0');
-		ptr++;
-		if (isdigit (*ptr)) {
-			timegda->timezone = timegda->minute * 10 + sign * (*ptr - '0');
-			ptr++;
-		}
-		if (*ptr)
-			return FALSE;
-		if ((timegda->timezone >= -24) && (timegda->timezone <= 24))
-			timegda->timezone *= 60*60;
-		else {
-			timegda->timezone = 0;
-			return FALSE;
-		}
-	}
-	return TRUE;
+	if (gda_parse_iso8601_time (timegda, value))
+		return TRUE;
+	else
+		return gda_parse_formatted_time (timegda, value, 0);
 }
 
 
 static GValue *
-gda_handler_time_get_sane_init_value (GdaDataHandler *iface, GType type)
+gda_handler_time_get_sane_init_value (G_GNUC_UNUSED GdaDataHandler *iface, GType type)
 {
-	GdaHandlerTime *hdl;
 	GValue *value = NULL;
 	
 	time_t now;
 	struct tm *stm;
 
 	g_return_val_if_fail (GDA_IS_HANDLER_TIME (iface), NULL);
-	hdl = (GdaHandlerTime*) (iface);
 
 	now = time (NULL);
 #ifdef HAVE_LOCALTIME_R
